@@ -1,3 +1,6 @@
+//go:build !remote
+// +build !remote
+
 package libpod
 
 import (
@@ -7,3 +10,7 @@ import (
 func (r *Runtime) platformMakePod(pod *Pod, resourceLimits *spec.LinuxResources) (string, error) {
 	return "", nil
 }
+
+func (p *Pod) removePodCgroup() error {
+	return nil
+}
