@@ -19,6 +19,7 @@ import (
 
 	"github.com/containers/buildah/define"
 	"github.com/containers/image/v5/types"
+	ldefine "github.com/containers/podman/v4/libpod/define"
 	"github.com/containers/podman/v4/pkg/auth"
 	"github.com/containers/podman/v4/pkg/bindings"
 	"github.com/containers/podman/v4/pkg/domain/entities"
@@ -26,6 +27,7 @@ import (
 	"github.com/containers/storage/pkg/fileutils"
 	"github.com/containers/storage/pkg/ioutils"
 	"github.com/containers/storage/pkg/regexp"
+	"github.com/docker/docker/pkg/jsonmessage"
 	"github.com/docker/go-units"
 	"github.com/hashicorp/go-multierror"
 	jsoniter "github.com/json-iterator/go"
@@ -39,6 +41,14 @@ type devino struct {
 
 var iidRegex = regexp.Delayed(`^[0-9a-f]{12}`)
 
+type BuildResponse struct {
+	Stream string                 `json:"stream,omitempty"`
+	Error  *jsonmessage.JSONError `json:"errorDetail,omitempty"`
+	// NOTE: `error` is being deprecated check https://github.com/moby/moby/blob/master/pkg/jsonmessage/jsonmessage.go#L148
+	ErrorMessage string          `json:"error,omitempty"` // deprecate this slowly
+	Aux          json.RawMessage `json:"aux,omitempty"`
+}
+
 // Build creates an image using a containerfile reference
 func Build(ctx context.Context, containerFiles []string, options entities.BuildOptions) (*entities.BuildReport, error) {
 	if options.CommonBuildOpts == nil {
@@ -393,6 +403,10 @@ func Build(ctx context.Context, containerFiles []string, options entities.BuildO
 		params.Add("unsetenv", uenv)
 	}
 
+	for _, ulabel := range options.UnsetLabels {
+		params.Add("unsetlabel", ulabel)
+	}
+
 	var (
 		headers http.Header
 	)
@@ -426,6 +440,11 @@ func Build(ctx context.Context, containerFiles []string, options entities.BuildO
 
 	dontexcludes := []string{"!Dockerfile", "!Containerfile", "!.dockerignore", "!.containerignore"}
 	for _, c := range containerFiles {
+		// Don not add path to containerfile if it is a URL
+		if strings.HasPrefix(c, "http://") || strings.HasPrefix(c, "https://") {
+			newContainerFiles = append(newContainerFiles, c)
+			continue
+		}
 		if c == "/dev/stdin" {
 			content, err := io.ReadAll(os.Stdin)
 			if err != nil {
@@ -496,6 +515,11 @@ func Build(ctx context.Context, containerFiles []string, options entities.BuildO
 		}
 	}
 
+	saveFormat := ldefine.OCIArchive
+	if options.OutputFormat == define.Dockerv2ImageManifest {
+		saveFormat = ldefine.V2s2Archive
+	}
+
 	// build secrets are usually absolute host path or relative to context dir on host
 	// in any case move secret to current context and ship the tar.
 	if secrets := options.CommonBuildOpts.Secrets; len(secrets) > 0 {
@@ -588,17 +612,13 @@ func Build(ctx context.Context, containerFiles []string, options entities.BuildO
 
 	var id string
 	for {
-		var s struct {
-			Stream string `json:"stream,omitempty"`
-			Error  string `json:"error,omitempty"`
-		}
-
+		var s BuildResponse
 		select {
 		// FIXME(vrothberg): it seems we always hit the EOF case below,
 		// even when the server quit but it seems desirable to
 		// distinguish a proper build from a transient EOF.
 		case <-response.Request.Context().Done():
-			return &entities.BuildReport{ID: id}, nil
+			return &entities.BuildReport{ID: id, SaveFormat: saveFormat}, nil
 		default:
 			// non-blocking select
 		}
@@ -612,7 +632,7 @@ func Build(ctx context.Context, containerFiles []string, options entities.BuildO
 			if errors.Is(err, io.EOF) && id != "" {
 				break
 			}
-			return &entities.BuildReport{ID: id}, fmt.Errorf("decoding stream: %w", err)
+			return &entities.BuildReport{ID: id, SaveFormat: saveFormat}, fmt.Errorf("decoding stream: %w", err)
 		}
 
 		switch {
@@ -622,15 +642,15 @@ func Build(ctx context.Context, containerFiles []string, options entities.BuildO
 			if iidRegex.Match(raw) {
 				id = strings.TrimSuffix(s.Stream, "\n")
 			}
-		case s.Error != "":
+		case s.Error != nil:
 			// If there's an error, return directly.  The stream
 			// will be closed on return.
-			return &entities.BuildReport{ID: id}, errors.New(s.Error)
+			return &entities.BuildReport{ID: id, SaveFormat: saveFormat}, errors.New(s.Error.Message)
 		default:
-			return &entities.BuildReport{ID: id}, errors.New("failed to parse build results stream, unexpected input")
+			return &entities.BuildReport{ID: id, SaveFormat: saveFormat}, errors.New("failed to parse build results stream, unexpected input")
 		}
 	}
-	return &entities.BuildReport{ID: id}, nil
+	return &entities.BuildReport{ID: id, SaveFormat: saveFormat}, nil
 }
 
 func nTar(excludes []string, sources ...string) (io.ReadCloser, error) {
@@ -695,15 +715,19 @@ func nTar(excludes []string, sources ...string) (io.ReadCloser, error) {
 					}
 					name = filepath.ToSlash(path)
 				}
-				excluded, err := pm.Matches(name) //nolint:staticcheck
-				if err != nil {
-					return fmt.Errorf("checking if %q is excluded: %w", name, err)
-				}
-				if excluded {
-					// Note: filepath.SkipDir is not possible to use given .dockerignore semantics.
-					// An exception to exclusions may include an excluded directory, therefore we
-					// are required to visit all files. :(
-					return nil
+				// If name is absolute path, then it has to be containerfile outside of build context.
+				// If not, we should check it for being excluded via pattern matcher.
+				if !filepath.IsAbs(name) {
+					excluded, err := pm.Matches(name) //nolint:staticcheck
+					if err != nil {
+						return fmt.Errorf("checking if %q is excluded: %w", name, err)
+					}
+					if excluded {
+						// Note: filepath.SkipDir is not possible to use given .dockerignore semantics.
+						// An exception to exclusions may include an excluded directory, therefore we
+						// are required to visit all files. :(
+						return nil
+					}
 				}
 				switch {
 				case dentry.Type().IsRegular(): // add file item
